/*
    Copyright (c) 2012 digitalSTROM.org, Zurich, Switzerland

    Author: Christian Hitz, aizo AG <christian.hitz@aizo.com>

    This file is part of po2json.

    po2json is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    po2json is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with po2json. If not, see <http://www.gnu.org/licenses/>.
*/

#include <string>
#include <fstream>
#include <iostream>
#include <stdlib.h>

using namespace std;

string quotedPart(string& line)
{
    int strStart = line.find("\"");
    if ((strStart == line.npos) ||
        (line[line.length() - 1] != '"') ||
        (strStart == (line.length() - 1))) {
        cerr << "quote error: " << line << endl;
        exit(1);
    }
    return line.substr(strStart + 1, line.length() - strStart - 2);
}

string splitHeader(string& line)
{
    int strStart = line.find("\"");
    if (strStart != 0) {
        cerr << "quote error: " << line << endl;
        exit(1);
    }
    int strEnd = line.find("\\n\"");
    if (strEnd == line.npos) {
        cerr << "quote error: " << line << endl;
        exit(1);
    }
    int colonPos = line.find(":");
    if (colonPos == line.npos) {
        cerr << "format error: " << line << endl;
        exit(1);
    }
    return line.substr(0, colonPos) + "\":\"" + line.substr(colonPos + 1, strEnd - colonPos - 1) + "\"";
}

typedef enum {
    eStart,
    eMessageId,
    eMessageIdFirst,
    eMessageString,
    eMessageStringContinue,
    eHeaderSkip,
    eHeader,
    eHeaderFirst,
    eFinish,
} parserState;

parserState poParser(parserState state, string& line)
{
    static string msgContext = "";
    if ((line.length() > 1) && (line[0] == '#')) {
        return state;
    }
    switch (state) {
    case eStart: {
        cout << "{";
        state = eMessageIdFirst;
        break;
    }
    case eMessageId:
        if (line.length() > 0) {
            cout << ",";
        }
        /* fall-through */
    case eMessageIdFirst: {
        if (line == "msgid \"\"") {
            state = eHeaderSkip;
        } else if (line.find("msgctxt") == 0) {
            msgContext = quotedPart(line) + '\004';
            state = eMessageIdFirst;
        } else if (line.find("msgid") == 0) {
            cout << '"' << msgContext << quotedPart(line) << "\":[";
            msgContext = "";
            state = eMessageString;
        }
        break;
    }
    case eMessageString: {
        if (line.find("msgid_plural ") == 0) {
            cout << '"' << quotedPart(line) << '"';
            state = eMessageStringContinue;
            break;
        } else if (line.find("msgstr") == 0) {
            cout << "null";
            /* fall-through */
        }
    }
    case eMessageStringContinue: {
        if (line.find("msgstr") == 0) {
            cout << ",\"" << quotedPart(line) << '"';
        } else if (line.length() == 0) {
            cout << "]";
            state = eMessageId;
            break;
        } else {
            cerr << "msgid is not followed by msgstr/msgid_plural" << endl;
            exit(1);
        }
        break;
    }
    case eHeaderSkip: {
        if (line == "msgstr \"\"") {
            state = eHeaderFirst;
            cout << "\"\":{";
        } else {
            cerr << "expected msgstr \"\": received instead: " << line << endl;
            exit(1);
        }
        break;
    }
    case eHeader:
        if (line.length() > 0) {
            cout << ",";
        }
        /* fall-through */
    case eHeaderFirst: {
        if (line[0] == '"') {
            cout << splitHeader(line);
            state = eHeader;
        } else {
            cout << "}";
            state = eMessageId;
        }
        break;
    }
    case eFinish: {
        cout << "}" << endl;
        break;
    }
    }
    return state;
}

int main(int argc, char *argv[])
{
    if (argc != 2) {
        cerr << "needs .po file as first argument" << endl;
        return 1;
    }
    ifstream ifs(argv[1]);
    string temp;

    parserState state = poParser(eStart, temp);
    while (getline(ifs, temp)) {
        state = poParser(state, temp);
    }
    poParser(eFinish, temp);
    return 0;
}
