Ext.namespace('dSS', 'dSS.data');

dSS.data.DeviceStore = Ext.extend(Ext.data.Store, {
    constructor: function(config) {
        // create a record constructor for device records
        var deviceRecord = Ext.data.Record.create([
        {
            name: "name"
        },
        {
            name: "dsid",
            mapping: "id"
        },
        {
            name: "blink",
            defaultValue: false
        },
        {
            name: "meterDSID"
        },
        {
            name: "modulatorName"
        },
        {
            name: "zoneId",
            mapping: "zoneID"

        },
        {
            name: "zoneName"
        },
        {
            name: "isPresent"
        },
        {
            name: "firstSeen"
        },
        {
            name: "inactiveSince",
            type: "date"
        },
        {
            name: "locked"
        },
        {
            name: "functionID"
        },
        {
            name: "comment"
        },
        {
            name: "productRevision"
        },
        {
            name: "productID"
        },
        {
            name: "HW"
        },
        {
            name: "firmwareVersion"
        },
        {
            name: "buttonID"
        },
        {
            name: "buttonInputMode"
        },
        {
            name: "buttonInputCount"
        },
        {
            name: "buttonInputIndex"
        },
        {
            name: "groups"
        },
        {
            name: "outputMode"
        }
        ]);

        // a json reader to read the device data
        var deviceReader = new Ext.data.JsonReader({
            root: "devices"
        },
        deviceRecord);

        Ext.apply(
        this, {
            reader: deviceReader,
            sortInfo: {
                field: 'firstSeen',
                direction: 'DESC' // or 'DESC' (case sensitive for local sorting)
            }
        });

        dSS.data.DeviceStore.superclass.constructor.call(this, arguments);

        this.on('load', function(store, records, options) {

            Ext.each(records, function(record) {
                var rev = record.get('productRevision');
                var major = rev >> 8;
                var mid = (rev & 0x00f0) >> 4;
                var last = rev & 0x000f;

                var hwString = dSS.util.deviceStringFromProduct(
                                                    record.get('functionID'),
                                                    record.get('productID'));

                record.set('HW', hwString);
                record.set('firmwareVersion', dSS.util.dec2hex(major) +
                           '.' + dSS.util.dec2hex(mid) +
                           '.' + dSS.util.dec2hex(last));
                record.commit();
             });

            Ext.Ajax.request({
                url: '/json/property/query',
                disableCaching: true,
                method: 'GET',
                scope: this,
                params: {
                    query: '/apartment/zones/zone0/devices/*(dSID,comment)'
                },
                success: function(result, request) {
                    try {
                        var json = Ext.util.JSON.decode(result.responseText);
                        if (json.ok) {
                            var modified = false;

                            Ext.each(json.result.devices, function(val) {
                                if (val.comment !== undefined) {
                                    var index = store.find("dsid",
                                              new RegExp("^" + val.dSID + "$"));
                                    if (index > - 1) {
                                        var rec = store.getAt(index);
                                        rec.set("comment", val.comment);
                                        modified = true;
                                    }
                                }
                            });
                            if (modified === true) {
                                store.commitChanges();
                            }
                        }
                    } catch (err) {
                        Ext.Msg.alert(dSS._("Error"),
                                      dSS._("Could not get comments from dSS") +
                                        ": " + err);
                    }
            },
            failure: function(result, request) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Failure when requesting comments from dSS"));
            }
        });

        },
        this);
    }
});

