//= require <dSS/data/ModulatorStore>
//= require <dSS/data/DeviceStore>
//= require <dSS/data/ZoneStore>
//= require <dSS/data/SceneStore>

Ext.namespace('dSS', 'dSS.data');

dSS.data.LoaderImpl = Ext.extend(Ext.util.Observable, {

    constructor: function() {
        this.modulatorStore = new dSS.data.ModulatorStore({
            url: '/json/apartment/getCircuits'
        });
        this.deviceStore = new dSS.data.DeviceStore();
        this.zoneStore = new dSS.data.ZoneStore();
        this.sceneStore = new dSS.data.SceneStore();

        this.modulatorStore.on('loadcomplete',
                               this.loadStructure.createDelegate(this));

        dSS.MessageBus.on('reloadapartment', this.load.createDelegate(this),
                           this);
    },

    load: function() {
        if (this.modulatorStore !== undefined) {
            this.modulatorStore.load();
        }
    },

    getModulatorStore: function() {
        return this.modulatorStore;
    },

    getDeviceStore: function() {
        return this.deviceStore;
    },

    getZoneStore: function() {
        return this.zoneStore;
    },

    getSceneStore: function() {
        return this.sceneStore;
    },


    loadStructure: function() {
        Ext.Ajax.request({
            url: '/json/apartment/getStructure',
            disableCaching: true,
            timeout: 300000,
            method: "GET",
            scope: this,
            success: function(result, request) {
                try {
                    var jsonData = Ext.util.JSON.decode(result.responseText);
                    this.processStructure(jsonData.result);
                    dSS.MessageBus.fireEvent('structureloaded');
                }
                catch(err) {
                    Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load apartment structure") + ": \"" + err + "\"");
                }
            },
            failure: function(result, request) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not load") + ": \"" + request.url + "\"");
            }
        });
    },

    processStructure: function(structure) {
        var devices = [],
        zones = [];
        var zoneStore = this.zoneStore;
        var modulatorStore = this.modulatorStore;
        var deviceStore = this.deviceStore;
        Ext.each(structure.apartment.zones, function(zone) {
            if (zone.id === 0) {
                zone.name = zone.name ? zone.name: dSS._("All devices");
                Ext.each(zone.devices, function(device) {
                    device.firstSeen = Date.parseDate(device.firstSeen,
                                                      "Y-m-d H:i:s", true);
                    device.inactiveSince = Date.parseDate(
                                                device.inactiveSince,
                                                "Y-m-d H:i:s", true);
                    devices.push(device);
                });
            }

            var unnamedInZone = false;
            Ext.each(zone.devices, function(device) {
                if ((device.name === undefined) || (device.name.length === 0)) {
                    unnamedInZone = true;
                }
                var i;
                for (i = 0; i < devices.length; i++) {
                    if (devices[i].id === device.id) {
                        devices[i].zoneId = zone.id;
                        devices[i].zoneName = zone.name;
                    }
                }
            });

            zones.push({
                id: zone.id,
                unnamedDevices: unnamedInZone,
                alternativeSort: zone.id === 0 ? 1 : 0,
                name: zone.name ? zone.name: dSS._("Zone") + ' #' + zone.id,
                primary: ('firstZoneOnDSMeter' in zone) ? true: false
            });

            zoneStore.loadData({
                zones: zones
            });
        });
        Ext.each(devices, function(device) {
            var index = modulatorStore.find('dsid',
                        new RegExp("^" + device.meterDSID + "$"));
            if (index > - 1) {
                var modulator = modulatorStore.getAt(index);
                device.modulatorName = modulator.get("name");
            }
        });
        deviceStore.loadData({
            devices: devices
        });
        modulatorStore.each(
            function(rec) {
                rec.set('numDevices',
                        deviceStore.queryBy(function(record, id) {
                            if (record.get('meterDSID') == rec.get('dsid')) {
                                return true;
                            }
                            return false;
                        }).length);
            });
        modulatorStore.commitChanges();
        this.sceneStore.loadScenes();
    }
});

