Ext.namespace("dSS", "dSS.config", "dSS.devprops");
// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.ConsumptionEventForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.Msg.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.deviceName = config.deviceName;
        this.eventIndex = config.eventIndex;

        this.addEvents('formsubmitted', 'formloaded', 'forcereload',
                       'saveenable');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'forcereload', 'saveenable' ],
            padding: 5,
            height: 100,
            scope: this,
            trackResetOnLoad: true,
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'checkbox',
                    ref: 'enableevent',
                    x: 6,
                    y: 9
                },
                {
                    xtype: 'label',
                    text: dSS.gt.strargs(dSS._("Activate consumption event '%1'"),
                            [(this.eventIndex == 0) ? dSS._("turn on") : dSS._("turn off")]),
                    x: 26,
                    y: 13
                },
                {
                    xtype: 'label',
                    ref: 'namelabel',
                    text: dSS._("Name") + ':',
                    disabled: true,
                    x: 26,
                    y: 42
                },
                {
                    xtype: 'textfield',
                    ref: 'name',
                    name: 'name',
                    allowBlank: false,
                    blankText: dSS._("Please enter a name for this consumption event"),
                    disabled: true,
                    width: 195,
                    x: 315,
                    y: 37
                },
                {
                    xtype: 'label',
                    ref: 'threaholdlabel',
                    text: dSS._("Send event if consumption of "),
                    disabled: true,
                    x: 26,
                    y: 71
                },
                {
                    xtype: 'numberfield',
                    ref: 'threshold',
                    name: 'threshold',
                    allowBlank: false,
                    allowDecimals: false,
                    allowNegative: false,
                    disabled: true,
                    minValue: 20,
                    minText: dSS._("The minmal threshold value is {0}"),
                    maxValue: 4095,
                    maxText: dSS._("The maximum threshold value is {0}"),
                    width: 60,
                    x: 315,
                    y: 66
                },
                {
                    xtype: 'label',
                    ref: 'tresholdunitlabel',
                    text: dSS._("W") + " " + ((this.eventIndex == 0) ? dSS._("overrun") : dSS._("underrun")),
                    disabled: true,
                    x: 380,
                    y: 71
                }
            ]
        });

        this.loaded = false;

        dSS.devprops.ConsumptionEventForm.superclass.constructor.apply(this, arguments);

        dSS.MessageBus.addListener('consumptionapplycurrent',
                          this.applyCurrent, this);

        this.enableevent.on('check', function(checkbox, checked) {
                if (checked === true) {
                    this.namelabel.enable();
                    this.name.enable();
                    this.name.validate();
                    this.threaholdlabel.enable();
                    this.threshold.enable();
                    this.threshold.validate();
                    this.tresholdunitlabel.enable();
                } else {
                    this.namelabel.disable();
                    this.name.disable();
                    this.threaholdlabel.disable();
                    this.threshold.disable();
                    this.tresholdunitlabel.disable();
                    this.fireEvent('saveenable', (1 << this.eventIndex * 2) +
                                                 (1 << this.eventIndex * 2 + 1), true);
                }
            },
            this);

        this.name.on('invalid', function(nameField, msg) {
                this.fireEvent('saveenable', (1 << this.eventIndex * 2), false);
            },
            this);

        this.name.on('valid', function(nameField, msg) {
                this.fireEvent('saveenable', (1 << this.eventIndex * 2), true);
            },
            this);

        this.threshold.on('invalid', function(nameField, msg) {
                this.fireEvent('saveenable', (1 << this.eventIndex * 2 + 1), false);
            },
            this);

        this.threshold.on('valid', function(nameField, msg) {
                this.fireEvent('saveenable', (1 << this.eventIndex * 2 + 1), true);
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                switch (data.loader) {
                    case dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_TABLE_ENTRY:
                        if (data.json.ok === true) {
                            if (data.json.result.eventIndex === this.eventIndex) {
                                this.loaded = true;
                                var name = data.json.result.eventName;
                                if (name === "") {
                                    if (this.deviceName === "") {
                                        name = dSS._("Device");
                                    } else {
                                        name = this.deviceName;
                                    }
                                    name += " " + ((this.eventIndex == 0) ? dSS._("turn on") : dSS._("turn off"));
                                }
                                this.name.setValue(name);
                                dSS.util.ResetDirtyField(this.name);
                                this.threshold.setValue(data.json.result.value);
                                dSS.util.ResetDirtyField(this.threshold);
                                this.enableevent.setValue(data.json.result.validity != 3);
                                this.enableevent.originalValue = this.enableevent.getValue();
                            }
                        }
                        break;

                    default:
                        return;
                }

                if (this.loaded === true) {
                    this.fireEvent('formloaded', true);

                    this.loaded = false;
                }
            },
            this);

        this.on('beforedestroy', function() {
                dSS.MessageBus.removeListener('consumptionapplycurrent',
                        this.applyCurrent, this);
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                if (this.getForm().isValid() !== true) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Please correct values in the red " +
                                        "marked fields and try again"));
                    return;
                }

                if (this.enableevent.isDirty() || this.name.isDirty() ||
                    this.threshold.isDirty()) {
                    if (this.enableevent.getValue()) {
                        dSS.util.genericRequest({
                            url: '/json/device/setSensorEventTableEntry',
                            scope: this,
                            params:
                            {
                                dsid: this.dsid,
                                eventIndex: this.eventIndex,
                                eventName: this.name.getValue(),
                                sensorIndex: 2,
                                test: ((this.eventIndex == 0) ? 2 : 1),
                                value: this.threshold.getValue(),
                                hysteresis: Math.round(this.threshold.getValue() / 2),
                                validity: 2,
                                action: 0
                            },
                            failureMsg: dSS._("Could not save sensor event table")
                        });
                    } else {
                        dSS.util.genericRequest({
                            url: '/json/device/setSensorEventTableEntry',
                            scope: this,
                            params:
                            {
                                dsid: this.dsid,
                                eventIndex: this.eventIndex,
                                eventName: "",
                                sensorIndex: 0,
                                test: 0,
                                value: 0,
                                hysteresis: 0,
                                validity: 3,
                                action: 0
                            },
                            failureMsg: dSS._("Could not save sensor event table")
                        });
                    }
                }
                this.fireEvent('formsubmitted');
            },
            this);
    },

    applyCurrent: function(currentValue) {
        this.threshold.setValue(Math.round(currentValue / 2));
    }
});

Ext.reg('consumptioneventform', dSS.devprops.ConsumptionEventForm);

dSS.devprops.CurrentConsumptionForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.Msg.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.consumptionValue = undefined;

        this.addEvents('formsubmitted', 'formloaded', 'forcereload');

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'forcereload' ],
            padding: 5,
            height: 69,
            scope: this,
            trackResetOnLoad: true,
            layout: 'absolute',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Current consumption") + ":",
                    x: 6,
                    y: 11
                },
                {
                    xtype: 'label',
                    ref: 'currentconsumption',
                    name: 'currentconsumption',
                    text: "",
                    x: 156,
                    y: 11
                },
                {
                    xtype: 'button',
                    text: dSS._("Refresh"),
                    ref: 'refreshconsumption',
                    handler: this.refreshConsumption.createDelegate(this),
                    minWidth: 95,
                    x: 315,
                    y: 7
                },
                {
                    xtype: 'button',
                    text: dSS._("Apply"),
                    ref: 'applyconsumption',
                    handler: this.applyConsumption.createDelegate(this),
                    minWidth: 95,
                    x: 416,
                    y: 7
                },
                {
                    xtype: 'label',
                    text: dSS._("It is recommended to set the threshold value to 50% of the maximal consumption."),
                    x: 6,
                    y: 38
                }
            ]
        });

        this.loaded = false;

        dSS.devprops.CurrentConsumptionForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                switch (data.loader) {
                    case dSS.devprops.Loaders.CONSUMPTION_TRESHOLD_CURRENT:
                        this.loaded = true;
                        if (data.json.ok === true) {
                            this.consumptionValue = data.json.result.sensorValue;
                            this.currentconsumption.setText(this.consumptionValue + " " + dSS._("W"));
                        } else {
                            this.consumptionValue = undefined;
                            this.currentconsumption.setText("--- " + dSS._("W"));
                        }
                        if (this.consumptionValue === undefined || this.consumptionValue < 40) {
                            this.applyconsumption.setDisabled(true);
                        }
                        break;

                    default:
                        return;
                }

                if (this.loaded === true) {
                    this.fireEvent('formloaded', true);

                    this.loaded = false;
                }
            },
            this);

        this.on('beforedestroy', function() {
                if (this.tId !== undefined) {
                    Ext.Ajax.abort(this.tId);
                }
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                this.fireEvent('formsubmitted');
            },
            this);
    },
    refreshConsumption: function() {
        // refresh current consumption
        this.tId = Ext.Ajax.request({
            url: '/json/device/getSensorValue',
            method: 'GET',
            params:
            {
                dsid: this.dsid,
                sensorIndex: 2
            },
            scope: this,
            success: function(response) {
                if (this.closed === true) {
                    return;
                }

                try {
                    var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                    if (jsonData.ok !== true) {
                        Ext.Msg.alert(dSS._("Error"),
                                      dSS._("Could not load current current " +
                                            "consumption") + ': ' +
                                            dSS._(jsonData.message));
                        return;
                    }

                    this.consumptionValue = jsonData.result.sensorValue
                    this.currentconsumption.setText(this.consumptionValue + " " + dSS._("W"));
                    this.applyconsumption.setDisabled(this.consumptionValue < 40);
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not load current current " +
                                        "consumption") + ': ' + err);

                    this.consumptionValue = undefined;
                    this.currentconsumption.setText("--- " + dSS._("W"));
                    this.applyconsumption.setDisabled(true);
                }
            },
            failure: function(response) {
                if (this.closed === true) {
                    return;
                }

                this.consumptionValue = undefined;
                this.currentconsumption.setText("--- " + dSS._("W"));
                this.applyconsumption.setDisabled(true);

                try {
                    var jsonData = Ext.util.JSON.decode(
                                                response.responseText);
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not load current current " +
                                        "consumption") + ': ' +
                                        dSS._(jsonData.message));
                } catch (err) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not load current current " +
                                        "consumption") + ': ' + err);
                }
            }
        });
    },
    applyConsumption: function() {
        if (this.consumptionValue !== undefined) {
            dSS.MessageBus.fireEvent('consumptionapplycurrent', this.consumptionValue);
        }
    }
});

Ext.reg('currentconsumptionform', dSS.devprops.CurrentConsumptionForm);
