Ext.namespace("dSS", "dSS.config", "dSS.devprops");
        // configuration values are set up and saved from the advanced form,
        // the standard form gives hints to the advanced form configuration

dSS.devprops.PBConfig = {
    "PUSHBUTTON_AREA1"  : -1,
    "PUSHBUTTON_AREA2"  : -2,
    "PUSHBUTTON_AREA3"  : -3,
    "PUSHBUTTON_AREA4"  : -4,
    "PUSHBUTTON_DEVICE" : -5,
    "PUSHBUTTON_ZONE"   : -6,
    "PUSHBUTTON_AREA"   : -7,
    "PUSHBUTTON_HIGHLEVEL" : -8
};

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid
dSS.devprops.GroupPushbuttonForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }
        this.dsid = config.dsid;
        this.group = config.group;
        this.product = config.product;

        // internal helper, this will never be displayed in the released
        // version
        if (config.matrix === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("GroupPushbuttonForm: " +
                                                  "missing matrix"));
            return;
        }
        this.matrix = config.matrix;

        this.addEvents('formsubmitted', 'formloaded');

        this.areaSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 105,
            x: 240,
            y: 25,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data:
                [
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA1, dSS._("Area 1")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA2, dSS._("Area 2")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA3, dSS._("Area 3")],
                    [dSS.devprops.PBConfig.PUSHBUTTON_AREA4, dSS._("Area 4")]
                ]
            }),
            valueField: 'id',
            displayField: 'displayText'
        });

        this.radio_pb_element = new Ext.form.Radio({
                boxLabel: dSS._("zone pushbutton"),
                name: 'rb_pushbutton_' + config.dsid,
                id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                inputValue : dSS.devprops.PBConfig.PUSHBUTTON_ZONE
            });

        this.radio_highlevel_element = new Ext.form.Radio({
                hidden: true,
                boxLabel: dSS._("server button"),
                name: 'rb_pushbutton_' + config.dsid,
                id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                inputValue : dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL
            });

        this.radio_areapb_element = new Ext.form.Radio({
            boxLabel: dSS._("area pushbutton"),
            name: 'rb_pushbutton_' + config.dsid,
            id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_AREA,
            inputValue : dSS.devprops.PBConfig.PUSHBUTTON_AREA
        });

        this.pbradio = new Ext.form.RadioGroup({
                    //xtype: 'radiogroup',
                    //ref: 'radio',
                    fieldLabel: dSS._("Pushbutton"),
                    width: 245,
                    columns: 1,
                    items:
                    [
                        {
                            disabled: !config.matrix.pushbdevice,
                            boxLabel: dSS._("device pushbutton"),
                            name: 'rb_pushbutton_' + config.dsid,
                            id: 'rb_pb_' + dSS.devprops.PBConfig.PUSHBUTTON_DEVICE,
                            inputValue : dSS.devprops.PBConfig.PUSHBUTTON_DEVICE
                        },
                        this.radio_areapb_element,
                        this.radio_pb_element,
                        this.radio_highlevel_element
                    ]
                });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 80,
                    width: 355,
                    layout: 'absolute',
                    items:
                    [
                        {
                            xtype: 'panel',
                            border: false,
                            width: 355,
                            x: 0,
                            y: 0,
                            layout: 'form',
                            items: [ this.pbradio ]
                        },
                        this.areaSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.GroupPushbuttonForm.superclass.constructor.apply(this, arguments);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.pbradio.markInvalid(data.json.message);
                    this.fireEvent('formloaded', true);
                    return;
                }

                var val = data.json.result[this.dsid][0].button[0].id;
                try {
                    var dec = this.decodeConfig(val);
                    this.pbradio.setValue('rb_pb_' + dec[0], dec[0]);
                    if ((this.matrix.highlevel === true) &&
                        (this.group == dSS.config.groups.JOKER) &&
                        ((dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) && 
                         (dec[0] != dSS.devprops.PBConfig.PUSHBUTTON_DEVICE))) {
                        dec[0] = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                        this.pbradio.setValue('rb_pb_' + dec[0], dec[0]);
                    }
                    if (dec[0] == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                        this.areaSelection.setValue(dec[1]);
                    } else if (dec[0] ==
                                dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                        this.radio_highlevel_element.show();
                        this.radio_pb_element.hide();
                        this.radio_areapb_element.disable();
                    }
                    if ((this.group == dSS.config.groups.JOKER) &&
                           ((this.product == dSS.config.deviceTypeNum['TKM']) || 
                            (this.product == dSS.config.deviceTypeNum['KL']) || 
                            (this.product == dSS.config.deviceTypeNum['ZWS']))) {
                            this.radio_highlevel_element.show();
                            this.radio_pb_element.hide();
                            this.radio_areapb_element.disable();
                    }
                } catch (ex) {
                    if (this.closed === true) {
                        return;
                    }

                    this.pbradio.markInvalid(ex);
                }
                this.fireEvent('formloaded', true);
            },
            this);

        if (this.matrix.highlevel === true) {
            dSS.MessageBus.addListener('jockerconfigformchanged',
                                  this.handleServerAppButton, this);
        }

        this.on('beforedestroy', function() {
                this.closed = true;
                if (this.matrix.highlevel === true) {
                    dSS.MessageBus.removeListener('jockerconfigformchanged',
                                           this.handleServerAppButton, this);
                }
            },
            this);

        this.areaSelection.on('select', function(combo, record, index) {
                var v = combo.getValue();
                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);

        this.pbradio.on('change', function(group, button) {
                var v = button.getRawValue();
                if (v == dSS.devprops.PBConfig.PUSHBUTTON_AREA) {
                    if (this.areaSelection.getRawValue() == "") {
                        this.areaSelection.setValue(dSS.devprops.PBConfig.PUSHBUTTON_AREA1);
                    }

                    v = this.areaSelection.getValue();
                    this.areaSelection.enable();
                } else {
                    this.areaSelection.disable();
                }
                if (this.firstLoad === true) {
                    this.firstLoad = false;
                    return;
                }

                dSS.MessageBus.fireEvent('pushbuttonstdformchanged', v);
            },
            this);

        this.on('submit', function() {
            // data will be saved by advanced form
            this.fireEvent('formsubmitted');
        },
        this);
    },

    handleServerAppButton: function(color) {
        if (this.matrix.highlevel !== true) {
            return;
        }

        var val = this.pbradio.getValue();

        if (color === dSS.config.groups.JOKER) {
            this.radio_highlevel_element.show();
            this.radio_pb_element.hide();

            if (val === this.radio_pb_element) {
                this.pbradio.setValue('rb_pb_' +
                            dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                            dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', true);
            } else if (val === this.radio_areapb_element) {
                if (this.matrix.pushbdevice === true) {
                    this.pbradio.setValue('rb_pb_' +
                            dSS.devprops.PBConfig.PUSHBUTTON_DEVICE,
                            dSS.devprops.PBConfig.PUSHBUTTON_DEVICE);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
                } else {
                    this.pbradio.setValue('rb_pb_' +
                            dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL,
                            dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL);
                    dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', true);
                }
            }
            this.radio_areapb_element.disable();
        } else {
            this.radio_pb_element.show();
            this.radio_highlevel_element.hide();
            this.radio_areapb_element.enable();

            if (this.pbradio.getValue() === this.radio_highlevel_element) {
                this.pbradio.setValue('rb_pb_' +
                            dSS.devprops.PBConfig.PUSHBUTTON_ZONE,
                            dSS.devprops.PBConfig.PUSHBUTTON_ZONE);
                dSS.MessageBus.fireEvent('pushbuttonhighlevelchanged', false);
            }
        }
    },

    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var rbval;
        var ext;
        switch (value)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA;
                ext = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");
        }
        return [ rbval, ext ];
    }
});

Ext.reg('dssdevgrouppushbuttonform', dSS.devprops.GroupPushbuttonForm);

// add advanced form, make sure it can communicate with the standard form

dSS.devprops.GroupPushbuttonAdvancedForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.matrix = config.matrix;
        this.group = config.group;

        this.addEvents('formsubmitted', 'formloaded');

        this.zonePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE0,
                dSS._("Scenes 0-4"),
                dSS._("Scenes 2-4"),
                dSS._("Scene 1")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE1,
                dSS._("Scenes 10-14"),
                dSS._("Scenes 12-14"),
                dSS._("Scene 11")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE2,
                dSS._("Scenes 20-24"),
                dSS._("Scenes 22-24"),
                dSS._("Scene 21")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE3,
                dSS._("Scenes 30-34"),
                dSS._("Scenes 32-34"),
                dSS._("Scene 31")
            ],
            [
                dSS.config.deviceParams.PUSHBUTTON_ZONE4,
                dSS._("Scenes 40-44"),
                dSS._("Scenes 42-44"),
                dSS._("Scene 41")
            ]
        ];

        this.areaPB =
        [
            // area 1
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP1,
                    dSS._("Scenes 2-4 (default)"),
                    dSS._("Scenes 2-4 (default)"),
                    dSS._("Area 1")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP5,
                    dSS._("Scenes 12-14"),
                    dSS._("Scenes 12-14"),
                    dSS._("Area 1")
                ]
            ],
            // area 2
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP2,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 2")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP6,
                    dSS._("Scenes 22-24"),
                    dSS._("Scenes 22-24"),
                    dSS._("Area 2")
                ]
            ],
            // area 3
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP3,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 3")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP7,
                    dSS._("Scenes 32-34"),
                    dSS._("Scenes 32-34"),
                    dSS._("Area 3")
                ]
            ],
            // area 4
            [
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP4,
                    dSS._("Scenes 2-4"),
                    dSS._("Scenes 2-4"),
                    dSS._("Area 4")
                ],
                [
                    dSS.config.deviceParams.PUSHBUTTON_GROUP8,
                    dSS._("Scenes 42-44"),
                    dSS._("Scenes 42-44"),
                    dSS._("Area 4")
                ]
            ]
        ];

        this.devicePB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_DEVICE,
                dSS._("Scenes 2-4"),
                dSS._("Scenes 2-4"),
                dSS._("Device")

            ]
        ];

        this.highlevelPB =
        [
            [
                dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL, "", "", ""
            ]
        ];

        this.sceneSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            autoSelect: false,
            editable: false,
            width: 170,
            mode: 'local',
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText',
                    'label2Text',
                    'label1Text'
                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            fieldLabel: dSS._("Used scenes")
        });

        this.label1 = new Ext.form.Label({
             xtype: 'label',
             fieldLabel: dSS._("1x Click")
         });

        this.label2 = new Ext.form.Label({
            xtype: 'label',
            fieldLabel: dSS._("2x, 3x, 4x Click")
        });

        this.localPrio = new Ext.form.Checkbox({
            boxLabel: dSS._("Local priority for 1x click")
        });

        this.lpTip = new Ext.ToolTip({
            html: dSS._("The local priority settings were changed."),
            title: dSS._("Notice"),
            closable: true,
            draggable: true,
            dismissDelay: 4000,
            anchor: 'left'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            border: false,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    ref: 'main',
                    height: 110,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        {
                            xtype: 'label',
                            text: dSS._("Pushbutton") + ':'
                        },
                        this.sceneSelection,
                        this.label1,
                        this.label2,
                        this.localPrio
                    ]
                }
            ]
        });

        dSS.devprops.GroupPushbuttonAdvancedForm.superclass.constructor.apply(this, arguments);


        dSS.MessageBus.addListener('pushbuttonstdformchanged',
                          this.updateForm, this);

        dSS.MessageBus.addListener('jockerconfigformchanged',
                          this.updateLocalPriority, this);

        this.sceneSelection.on('select', function(combo, record, index) {
                this.label1.setText(record.get('label1Text'));
                this.label2.setText(record.get('label2Text'));
            },this);

        this.loadedValue = undefined;
        this.loadedLocalPrio = undefined;
        this.currentValue = undefined;

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader == dSS.devprops.Loaders.COMBINED_PB_OUTMODE) {
                    if (data.json.ok !== true) {
                        this.sceneSelection.markInvalid(data.json.message);
                        this.label1.setText("");
                        this.label2.setText("");
                        this.localPrio.setDisabled(true);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    try {
                        var val = data.json.result[this.dsid][0].button[0].id;
                        var prio = data.json.result[this.dsid][0].button[0].setsLocalPriority;

                        var load = this.decodeConfig(val);
                        this.loadedValue = load[1];
                        if ((this.matrix.highlevel === true) &&
                            (this.group == dSS.config.groups.JOKER) &&
                            (((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL) &&
                             ((val & 0xf) != dSS.config.deviceParams.PUSHBUTTON_DEVICE))) {
                            val = dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL;
                            load = this.decodeConfig(val);
                        }
                        this.sceneSelection.getStore().loadData(load[0]);
                        this.sceneSelection.setValue(load[1]);
                        this.label1.setText(load[2]);
                        this.label2.setText(
                            this.sceneSelection.getStore().getById(load[1]).get(
                                                                    'displayText'));
                        this.currentValue = load[4];
                        this.loadedValueLP = prio;
                        this.localPrio.setValue(prio);
                        this.localPrio.setDisabled(!load[3]);

                        if (load[4] ==
                                dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                            this.disableElements();
                        }
                    } catch (ex) {
                        if (this.closed === true) {
                            return;
                        }

                        this.sceneSelection.markInvalid(ex);
                        this.label1.setText("");
                        this.label2.setText("");
                    }
                } else if (data.loader == dSS.devprops.Loaders.SCENE_CONFIGURATION) {
                    if (data.json.ok !== true) {
                        this.sceneSelection.markInvalid(data.json.message);
                        this.label1.setText("");
                        this.label2.setText("");
                        this.localPrio.setDisabled(true);
                        this.fireEvent('formloaded', true);
                        return;
                    }

                    try {
                        this.loadedLocalPrio = data.json.result.specialMode;
                        if (this.loadedValue === dSS.config.deviceParams.PUSHBUTTON_DEVICE) {
                            this.localPrio.setValue((this.loadedLocalPrio) ? 1 : 0);
                            this.localPrio.setDisabled(false);
                        }
                    } catch (ex) {
                        if (this.closed === true) {
                            return;
                        }
                    }
                }

                if ((this.loadedValue !== undefined) &&
                    (this.loadedLocalPrio !== undefined)) {
                    this.fireEvent('formloaded', true);
                }
            },
            this);

        this.on('beforedestroy', function() {
                dSS.MessageBus.removeListener('pushbuttonstdformchanged',
                                              this.updateForm, this);
                dSS.MessageBus.removeListener('jockerconfigformchanged',
                                              this.updateLocalPriority, this);
                this.closed = true;
            },
            this);

        this.on('submit', function() {
            var value = this.sceneSelection.getValue();
            var prio = this.localPrio.getValue();

            if (!(value !== this.loadedValue) &&
                !((value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedLocalPrio)) &&
                !((value !== dSS.config.deviceParams.PUSHBUTTON_DEVICE) && (prio != this.loadedValueLP))) {
                this.fireEvent('formsubmitted');
                return;
            }

            var id = this.dsid;
            var deferDelay = 1000;

            if (value !== this.loadedValue) {
                // we know this form is never used alone, it's always in the
                // same dialog as the output mode form; in order to not spam
                // the bus with two simultanious requests we will delay saving
                // of pushbutton settings by one second
                dSS.util.genericRequest.defer(deferDelay,
                    this,
                    [
                        {
                            url: '/json/device/setButtonID',
                            params:
                            {
                                dsid : this.dsid,
                                buttonID : value
                            },
                            failureMsg:
                                    dSS._("Could not save pushbutton settings"),
                            scope: this,
                            evt: 'formsubmitted'
                        },
                        function(success) {
                            if (success === true) {
                                var s = dSS.data.Loader.getDeviceStore();
                                var rec = s.getById(id);
                                if (rec !== undefined) {
                                    rec.set('buttonID', value);
                                    rec.commit();
                                    dSS.MessageBus.fireEvent('scenesmarkdirty');
                                }
                            }
                        }
                    ]
                );
            }

            if (value === dSS.config.deviceParams.PUSHBUTTON_DEVICE) {
                if (prio != this.loadedLocalPrio) {
                    deferDelay += 500;
                    dSS.util.genericRequest.defer(deferDelay,
                        this,
                        [
                            {
                                url: '/json/device/setSceneMode',
                                params:
                                {
                                    sceneID : dSS.config.scenes.LOCAL_ON,
                                    specialMode: prio ? 1 : 0,
                                    dsid: this.dsid
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings"),
                                scope: this,
                                evt: 'formsubmitted'
                            }
                        ]
                    );
                }
            } else {
                if (prio != this.loadedValueLP) {
                    deferDelay += 500;
                    dSS.util.genericRequest.defer(deferDelay,
                        this,
                        [
                            {
                                url: '/json/property/setBoolean',
                                params:
                                {
                                    path: '/apartment/zones/zone0/devices/' +
                                          this.dsid +
                                          '/button/setsLocalPriority',
                                    value: prio
                                },
                                failureMsg:
                                        dSS._("Could not save pushbutton settings"),
                                scope: this,
                                evt: 'formsubmitted'
                            }
                        ]
                    );
                }
            }
        },
        this);
    },
    // decode pushbutton settings value, bank 3 offset 1
    decodeConfig: function(value) {
        var ret;
        var l1;
        var showLocalPriority = false;
        var rbval;
        switch (value)
        {
            case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
                ret = this.devicePB;
                l1 = dSS._("Device");
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_DEVICE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
                ret = this.areaPB[0];
                l1 = dSS._("Area 1");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA1;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
                ret = this.areaPB[1];
                l1 = dSS._("Area 2");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA2;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
                ret = this.areaPB[2];
                l1 = dSS._("Area 3");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA3;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
                ret = this.areaPB[3];
                l1 = dSS._("Area 4");
                showLocalPriority = true;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_AREA4;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
                l1 = this.zonePB[0][3];
            case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
                if (l1 === undefined) {
                    l1 = this.zonePB[1][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
                if (l1 === undefined) {
                    l1 = this.zonePB[2][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
                if (l1 === undefined) {
                    l1 = this.zonePB[3][3];
                }
            case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
                if (l1 === undefined) {
                    l1 = this.zonePB[4][3];
                }
                ret = this.zonePB;
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_ZONE;
                break;
            case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
                rbval = dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL;
                ret = this.highlevelPB;
                l1 = "";
                break;
            default:
                throw dSS._("Unknown pushbutton configuration!");

        }
        return [ ret, value, l1, showLocalPriority, rbval ];
    },
    updateLocalPriority: function(color) {
        if ((this.group != color) &&
            (color != dSS.config.groups.JOKER) &&
            (this.group != dSS.config.groups.JOKER) &&
            ((this.group == dSS.config.groups.SHADOW) ||
             (color == dSS.config.groups.SHADOW))) {
            this.group = color;
            if (this.updateLocalPrio()) {
                if ((this.ownerCt !== undefined) &&
                    (this.ownerCt.xtype === 'panel') &&
                    (this.ownerCt.collapsed === true)) {
                        this.lpTip.initTarget(this.ownerCt.getId());
                        this.lpTip.enable();
                        this.lpTip.show();
                }
            }
        } else {
            this.group = color;
            // Hide tooltip again
            if ((this.ownerCt !== undefined) &&
                (this.ownerCt.xtype === 'panel')) {
                    this.lpTip.disable();
                    this.lpTip.hide();
            }
        }
    },
    enableElements: function() {
        this.sceneSelection.enable();
        this.label1.enable();
        this.label2.enable();
    },
    disableElements: function() {
        this.sceneSelection.disable();
        this.label1.disable();
        this.label2.disable();
    },
    updateForm: function(value) {
        this.currentValue = value;
        if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA1) {
                this.sceneSelection.getStore().loadData(this.areaPB[0]);
                this.sceneSelection.setValue(this.areaPB[0][0][0]);
                this.label1.setText(dSS._("Area 1"));
                this.label2.setText(this.areaPB[0][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA2) {
                this.sceneSelection.getStore().loadData(this.areaPB[1]);
                this.sceneSelection.setValue(this.areaPB[1][0][0]);
                this.label1.setText(dSS._("Area 2"));
                this.label2.setText(this.areaPB[1][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA3) {
                this.sceneSelection.getStore().loadData(this.areaPB[2]);
                this.sceneSelection.setValue(this.areaPB[2][0][0]);
                this.label1.setText(dSS._("Area 3"));
                this.label2.setText(this.areaPB[2][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA4) {
                this.sceneSelection.getStore().loadData(this.areaPB[3]);
                this.sceneSelection.setValue(this.areaPB[3][0][0]);
                this.label1.setText(dSS._("Area 4"));
                this.label2.setText(this.areaPB[3][0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
                this.sceneSelection.getStore().loadData(this.devicePB);
                this.sceneSelection.setValue(this.devicePB[0][0]);
                this.label1.setText(dSS._("Device"));
                this.label2.setText(this.devicePB[0][1]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
                this.sceneSelection.getStore().loadData(this.zonePB);
                this.sceneSelection.setValue(this.zonePB[0][0]);
                this.label1.setText(this.zonePB[0][3]);
                this.label2.setText(this.zonePB[0][2]);
                this.enableElements();
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_HIGHLEVEL) {
                this.sceneSelection.getStore().loadData(this.highlevelPB);
                this.sceneSelection.setValue(this.highlevelPB[0][0]);
                this.label1.setText("");
                this.label2.setText("");
                this.disableElements();
        } else {
                this.sceneSelection.getStore().loadData([]);
                this.sceneSelection.setValue();
                this.sceneSelection.markInvalid(
                        dSS._("Unknown pushbutton configuration!"));
                this.label1.setText("");
                this.label2.setText("");
        }
        this.updateLocalPrio();
    },
    updateLocalPrio: function() {
        var value = this.currentValue;
        if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA1) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA2) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA3) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_AREA4) {
            this.localPrio.setDisabled(false);
            if (this.group == dSS.config.groups.SHADOW) {
                this.localPrio.setValue(0);
            } else {
                this.localPrio.setValue(1);
            }
            return true;
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_DEVICE) {
            this.localPrio.setDisabled(false);
            this.localPrio.setValue((this.loadedLocalPrio) ? 1 : 0);
        } else if (value == dSS.devprops.PBConfig.PUSHBUTTON_ZONE) {
            this.localPrio.setDisabled(true);
            this.localPrio.setValue(0);
        } else {
            this.localPrio.setDisabled(true);
        }
        return false;
    }
});

Ext.reg('dssdevgrouppushbuttonadvancedform', dSS.devprops.GroupPushbuttonAdvancedForm);

