//= require <external/IconCombo>

Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid, group (configured joker group)
dSS.devprops.JokerConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.group = config.group;

        this.addEvents('formsubmitted', 'formloaded');

        this.colorSelection = new Ext.ux.IconCombo({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 130,
            mode: 'local',
            fieldLabel: dSS._("Color group"),
            tpl: new Ext.XTemplate(
                '<tpl for="."><div class="x-combo-list-item">',
                '<table><tbody><tr>',
                '<td>',
                '<div class="{css} x-icon-combo-icon"></div></td>',
                '<td>{displayText}</td>',
                '</tr></tbody></table>',
                '</div></tpl>'),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'css',
                    'displayText'
                ],
                data:
                [
                    [
                        dSS.config.groups.LIGHT,
                        dSS.config.GroupColorIconCls[dSS.config.groups.LIGHT],
                        dSS._("Light")
                    ],
                    [
                        dSS.config.groups.SHADOW,
                        dSS.config.GroupColorIconCls[dSS.config.groups.SHADOW],
                        dSS._("Shadow")
                    ],
                    [
                        dSS.config.groups.CLIMATE,
                        dSS.config.GroupColorIconCls[dSS.config.groups.CLIMATE],
                        dSS._("Climate")
                    ],
                    [
                        dSS.config.groups.AUDIO,
                        dSS.config.GroupColorIconCls[dSS.config.groups.AUDIO],
                        dSS._("Audio")
                    ],
                    [
                        dSS.config.groups.VIDEO,
                        dSS.config.GroupColorIconCls[dSS.config.groups.VIDEO],
                        dSS._("Video")
                    ],
                    [
                        dSS.config.groups.SECURITY,
                        dSS.config.GroupColorIconCls[
                                                dSS.config.groups.SECURITY],
                        dSS._("Security")
                    ],
                    [
                        dSS.config.groups.ACCESS,
                        dSS.config.GroupColorIconCls[dSS.config.groups.ACCESS],
                        dSS._("Access")
                    ],
                    [
                        dSS.config.groups.JOKER,
                        dSS.config.GroupColorIconCls[dSS.config.groups.JOKER],
                        dSS._("Joker")
                    ]

                ]
            }),
            valueField: 'id',
            displayField: 'displayText',
            iconClsField: 'css'
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.colorSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.JokerConfigForm.superclass.constructor.apply(this, arguments);

        this.on('afterrender', function() {
                this.fireEvent('formloaded', true);
            },
            this);

        this.colorSelection.on('afterrender', function() {
                if ((this.closed !== true) &&
                   ((this.group >= dSS.config.groups.LIGHT) &&
                    (this.group <= dSS.config.groups.JOKER))) {
                    this.colorSelection.setValue(this.group);
                }
            },
            this);

        this.colorSelection.on('select', function(box, record, index) {
                dSS.MessageBus.fireEvent('jockerconfigformchanged', record.data.id);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                var value = this.colorSelection.getValue();
                var dsid = this.dsid;
                var group = this.group;
                if (value == group) {
                    this.fireEvent('formsubmitted');
                    return;
                }

                dSS.util.genericRequest({
                    url: '/json/device/setJokerGroup',
                    params:
                    {
                        dsid : this.dsid,
                        groupID : value
                    },
                    failureMsg:
                        dSS._("Could not set joker device group"),
                    scope: this,
                    evt: 'formsubmitted'
                },
                function(success) {
                    if (success === true) {
                        var devStore = dSS.data.Loader.getDeviceStore();
                        var record = devStore.getById(dsid);
                        if (record !== undefined) {
                            var groups = dSS.util.updateJokerGroup(
                                            record.get('groups'), group, value);
                            record.set('groups', groups);
                            record.commit();
                        }
                        dSS.MessageBus.fireEvent('scenesmarkdirty');
                    }
                });
        },
        this);
    }
});

Ext.reg('dssdevjokerconfigform', dSS.devprops.JokerConfigForm);

