Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dsid, group (configured joker group)
dSS.devprops.MultiButtonConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        if (config.dsid === undefined) {
            Ext.MessageBox.alert(dSS._("Error"), ("Missing dSID"));
            return;
        }

        this.dsid = config.dsid;
        this.buttonConfig = config.buttonConfig;
        this.buttonIndex = config.buttonIndex;
        this.buttonID = config.buttonID;
        this.group = config.group;
        this.matrix = config.matrix;

        this.addEvents('formsubmitted', 'formloaded');

        var selectionItems =
            [
                [
                    dSS.config.ButtonInputMode.BUTTONINPUT_1WAY,
                    dSS._("1-way")
                ],
                [
                    dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN,
                    dSS._("2-way down")
                ],
                [
                    dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP,
                    dSS._("2-way up")
                ]
            ];

        if (this.matrix.pushbcombined === true) {
            selectionItems =
                [
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY_COMBINED,
                        dSS._("1-way")
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_2WAY,
                        dSS._("2-way")
                    ],
                    [
                        dSS.config.ButtonInputMode.BUTTONINPUT_1WAY,
                        dSS._("1-way + joker")
                    ]
                ];
        }

        this.buttonInputTypeSelection = new Ext.form.ComboBox({
            typeAhead: false,
            triggerAction: 'all',
            lazyRender:true,
            forceSelection: true,
            editable: false,
            width: 160,
            mode: 'local',
            fieldLabel: dSS._("Button function"),
            store: new Ext.data.ArrayStore({
                id: 0,
                fields:
                [
                    'id',
                    'displayText'
                ],
                data: selectionItems
            }),
            valueField: 'id',
            displayField: 'displayText',
            disabled: (this.buttonID === dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL) ||
                      (this.group == dSS.config.groups.JOKER)
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'panel',
                    border: false,
                    height: 22,
                    width: 355,
                    layout: 'form',
                    items:
                    [
                        this.buttonInputTypeSelection
                    ]
                }
            ]
        });

        this.firstLoad = true;

        dSS.devprops.MultiButtonConfigForm.superclass.constructor.apply(this, arguments);

        dSS.MessageBus.addListener('pushbuttonhighlevelchanged',
                          this.highlevelChanged, this);

        this.on('afterrender', function() {
                this.fireEvent('formloaded', true);
            },
            this);
        
        this.translateButtonconfigToSelection = function(buttonConfig) {
            if ((buttonConfig >=
                    dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT1) &&
                (buttonConfig <=
                    dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT4)) {
                 return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_DOWN;
            } else if ((buttonConfig >=
                     dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT1) &&
                       (buttonConfig <=
                     dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT4)) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY_UP;
            } else if (buttonConfig ==
                                dSS.config.deviceParams.BUTTONINPUT_2WAY) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_2WAY;
            } else if (buttonConfig ==
                                dSS.config.deviceParams.BUTTONINPUT_1WAY) {
                return dSS.config.ButtonInputMode.BUTTONINPUT_1WAY_COMBINED;
            } else {
                return dSS.config.ButtonInputMode.BUTTONINPUT_1WAY;
            }
        };

        this.buttonInputTypeSelection.on('afterrender', function() {
                if (this.closed !== true) {
                    this.buttonInputTypeSelection.setValue(this.translateButtonconfigToSelection(this.buttonConfig));
                }
            },
            this);

        this.on('beforedestroy', function() {
                dSS.MessageBus.removeListener('pushbuttonhighlevelchanged',
                        this.highlevelChanged, this);
                this.closed = true;
            },
            this);

        this.on('submit', function() {
                var value = this.buttonInputTypeSelection.getValue();
                var dsid = this.dsid;
                var buttonConfig = this.buttonConfig;
                if (value == this.translateButtonconfigToSelection(buttonConfig)) {
                    this.fireEvent('formsubmitted');
                    return;
                }
                
                Ext.Ajax.request({
                    url: '/json/device/setButtonInputMode',
                    method: 'GET',
                    disableCaching: true,
                    scope: this,
                    params:
                    {
                        dsid : dsid,
                        mode : value
                    },
                    success: function(result, request) {
                        try {
                            var jsonData = Ext.util.JSON.decode(result.responseText);
                            if (jsonData.ok !== true) {
                                throw "error";
                            }

                            var reply = jsonData.result;
                            var store = dSS.data.Loader.getDeviceStore();

                            var index = store.find('dsid', reply.update.dsid);
                            if (index > -1) {
                                var d = store.getAt(index);
                                d.set('buttonInputMode',
                                        reply.update.buttonInputMode);
                                d.commit();
                            }

                            var dsm;
                            var ms = dSS.data.Loader.getModulatorStore();
                            index = ms.find('dsid', reply.device.meterDSID);
                            if (index > - 1) {
                                dsm = ms.getAt(index);
                            }

                            if (reply.action === "add") {

                               reply.device.firstSeen = Date.parseDate(
                                    reply.device.firstSeen, "Y-m-d H:i:s",
                                    true);
                                reply.device.inactiveSince = Date.parseDate(
                                    reply.device.inactiveSince, "Y-m-d H:i:s",
                                    true);

                                if (dsm !== undefined) {
                                    reply.device.modulatorName = dsm.get("name");
                                    dsm.set('numDevices',
                                            dsm.get('numDevices') + 1);
                                }

                                var dev = { devices : [] };
                                dev.devices.push(reply.device);
                                store.loadData(dev, true);
                                dSS.MessageBus.fireEvent('scenesmarkdirty');
                            } else if (reply.action === "remove") {
                                 index = store.find('dsid', reply.device.id);
                                 if (index > -1) {
                                     store.removeAt(index);
                                     dSS.MessageBus.fireEvent('scenesmarkdirty');
                                 }
                                 if (dsm !== undefined) {
                                    dsm.set('numDevices',
                                            dsm.get('numDevices') - 1);
                                 }
                            }
                            dSS.MessageBus.fireEvent('scenesmarkdirty');
                            this.fireEvent('formsubmitted', true);
                        } catch (err) {
                            Ext.Msg.alert(dSS._("Error"),
                                dSS._("Could not set button input mode " +
                                      "of device"));
                            this.fireEvent('formsubmitted', false);
                        }
                    },
                    failure: function(result, request) {
                        Ext.Msg.alert(dSS._("Error"),
                            dSS._("Could not set button input mode " +
                                  "of device"));
                        this.fireEvent('formsubmitted', false);
                    }
                });
        },
        this);
    },
    highlevelChanged: function(highlevelActive) {
        if (highlevelActive) {
            this.buttonInputTypeSelection.setValue(dSS.config.ButtonInputMode.BUTTONINPUT_1WAY);
        }
        this.buttonInputTypeSelection.setDisabled(highlevelActive);
    }
});

Ext.reg('dssdevmultibuttonconfigform', dSS.devprops.MultiButtonConfigForm);

