Ext.namespace("dSS", "dSS.util");

dSS.util.shortDSID = function(dsid) {
    if (dsid.length > 8) {
        return dsid.slice(dsid.length - 8);
    } else {
        return dsid;
    }
};

dSS.util.getDSIDPlusOne = function(dsid) {
    var incShortDSID = parseInt(dSS.util.shortDSID(dsid), 16) + 1;
    var shortDSIDString = incShortDSID.toString(16);
    while (shortDSIDString.length < 8) {
        shortDSIDString = "0" + shortDSIDString;
    }
    return dsid.slice(0, dsid.length - 8) + shortDSIDString;
};

dSS.util.getName = function(name) {
    if ((name === undefined) || (name.length === 0)) {
        return '<i>' + dSS._("no name") + '</i>';
    } else {
        return name;
    }
};

dSS.util.getNameOrDSID = function(name, dsid) {
    if ((name === undefined) || (name.length === 0)) {
        return dSS.util.shortDSID(dsid);
    } else {
        return name;
    }
};

dSS.util.decode16 = function(version) {
    var low =  (version & 0x000000ff);
    var high = (version & 0x0000ff00) >> 8;
    return { high: high, low: low };
};

dSS.util.decode16V = function(version) {
    var ret = dSS.util.decode16(version);
    return String(ret.high) + "." + ret.low;
};

dSS.util.decode32 = function(version) {
    var v1 = (version & 0xff000000) >> 24;
    var v2 = (version & 0x00ff0000) >> 16;
    var v3 = (version & 0x0000ff00) >> 8;
    var v4 = (version & 0x000000ff);

    return String(v1) + "." + v2  + "." + v3 + "." + v4;
};

dSS.util.parseFunctionID = function(functionID) {
    var dSclass = (functionID & 0xf000) >> 12;
    var dSsubclass = (functionID & 0xfc0) >> 6;
    var dSfuncModule = (functionID & 0x3f);

    return [dSclass, dSsubclass, dSfuncModule];
};

dSS.util.deviceHasMultipleButtons = function(functionID) {
    var cls = dSS.util.parseFunctionID(functionID);

    if ((cls[1] === 0x04) && ((cls[2] & 0x3) > 1)) {
        return true;
    }
    return false;
};

dSS.util.deviceHasOutput = function(functionID) {
    var rec = dSS.util.parseFunctionID(functionID);

    return ((rec[2] & 0x10) == 0x10);
};

dSS.util.isAreaScene = function(scene) {
    if ((scene < dSS.config.scenes.T1_S0) ||
        (scene > dSS.config.scenes.T4_S1) ||
        (scene == dSS.config.scenes.T0_S1)) {
        return false;
    }
    return true;
};

dSS.util.parseProductID = function(productID) {
    var deviceType = (productID >> 10) & 0x3f;
    var deviceNumber = productID & 0x3ff;
    return [deviceType, deviceNumber];
};

dSS.util.deviceStringFromProduct = function(functionID, productID) {
    var fid = dSS.util.parseFunctionID(functionID);
    var devShort = dSS.config.deviceColors[fid[0]];
    if (devShort === undefined) {
        devShort = "";
    } else {
        devShort = devShort + "-";
    }

    var result = dSS.util.parseProductID(productID);
    if (result.length < 2) {
        return "";
    }

    var devType = dSS.config.deviceTypes[result[0]];
    if (devType === undefined) {
        devType = "??";
    }

    var devNumber = result[1];
    if (devNumber === undefined) {
        devNumber = "???";
    }

    return devShort + devType + " " + result[1];

};

dSS.util.dec2hex = function(d) {
  var hD = '0123456789ABCDEF';
  var h = hD.substr(d & 15, 1);
  while (d > 15) {
    d>>=4;
    h = hD.substr(d & 15, 1) + h;
  }
  return h;
};

dSS.util.callScene = function(dsid, sceneNumber) {
    Ext.Ajax.request({
        url: '/json/device/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            sceneNumber: sceneNumber
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"), dSS._("Could not call scene")
                                  + ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.callSceneOnZone = function(zoneId, sceneNumber, groupId, force) {
    if (force === undefined) {
        force = false;
    }
    Ext.Ajax.request({
        url: '/json/zone/callScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId,
            force: force
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"), dSS._("Could not call scene")
                                  + ': ' + jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send callScene command to dSS"));
        }
    });
};

dSS.util.saveSceneOnZone = function(zoneId, sceneNumber, groupId) {
    Ext.Ajax.request({
        url: '/json/zone/saveScene',
        disableCaching: true,
        method: "GET",
        params: {
            id: zoneId,
            sceneNumber: sceneNumber,
            groupID: groupId
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.MessageBox.alert(dSS._("Could not save scene") + " " + sceneNumber, jsonData.message);
                } else {
                    var stmp = dSS._(dSS.config.sceneNames[sceneNumber]);
                    var ztmp = dSS._("in zone") + ' &quot;' + '#' + zoneId +
                               '&quot; ';

                    var s = dSS.data.Loader.getSceneStore();
                    var srec = s.getById(zoneId + '-' + groupId + '-' +
                                         sceneNumber);
                    if (srec !== undefined) {
                        stmp = dSS.util.getName(srec.get('name'));
                    }

                    if (zoneId === 0) {
                        ztmp = '';
                    } else {
                        var z = dSS.data.Loader.getZoneStore();
                        var zrec = z.getById(zoneId);
                        if (zrec !== undefined) {
                            ztmp = dSS._("in zone") + ' &quot;' +
                                   dSS.util.getName(zrec.get('name')) +
                                   '&quot; ';
                        }
                    }

                    var msg = dSS._("Scene") + ' &quot;' + stmp + '&quot; ' +
                              ztmp +  dSS._("was saved");

                    dSS.MessageBus.fireEvent('statusmessage', msg);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send saveScene command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send saveScene command to dSS"));
        }
    });
};

dSS.util.setValueOnDevice = function(dsid, value) {
    Ext.Ajax.request({
        url: '/json/device/setValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            value: value
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not set value") + ": " +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send setValue command to dSS") + ": " + err);
            }
        },
        failure: function(result, request) {
            Ext.MessageBox.alert(dSS._("Error"), dSS._("Could not send setValue command to dSS"));
        }
    });
};

dSS.util.setOutputValueOnDevice = function(dsid, offset, value) {
    Ext.Ajax.request({
        url: '/json/device/setOutputValue',
        disableCaching: true,
        method: "GET",
        params: {
            dsid: dsid,
            offset: offset,
            value: value
        },
        success: function(result, request) {
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (!jsonData.ok) {
                    Ext.Msg.alert(dSS._("Error"),
                                  dSS._("Could not set value") + ": " +
                                  jsonData.message);
                }
            } catch(err) {
                Ext.Msg.alert(dSS._("Error"),
                    dSS._("Could not send setOutputValue command to dSS") +
                          ": " + err);
            }
        },
        failure: function(result, request) {
            var msg = '';
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (jsonData.message) {
                    msg = ': ' + dSS._(jsonData.message);
                }
            } catch (err) {
                msg = ': ' + err;
            }
            Ext.Msg.alert(dSS._("Error"),
                        dSS._("Could not send setOutputValue command to dSS") +
                        msg);
        }
    });
};


/* parameter object
    {
        url, // url string
        params // ajax request parameters
        failureMsg // failure message to show in alert messagebox
        maskEl // element to mask during loading
        maskMsg // mask message
        scope // object scope (passt to ajax requuest)
        evt // event name (string) to fire
    }
    returns AjaxRequest object cotnainig the transaction id
*/
dSS.util.genericRequest = function(param, callback) {
    if (param === undefined) {
        throw dSS._("Call with undefined parameter");
    }

    if (param.maskEl !== undefined) {
        if (param.maskMsg !== undefined) {
            param.maskEl.mask(param.maskMsg + '...', 'x-mask-loading');
        } else {
            param.maskEl.mask();
        }
    }

    return Ext.Ajax.request(
    {
        url: param.url,
        method: 'GET',
        params: param.params,
        scope: param.scope,
        success: function(response) {
            var ok = false;
            try {
                var jsonData = Ext.util.JSON.decode(
                                    response.responseText);
                if (jsonData.ok !== true) {
                    if (param.failureMsg !== undefined) {
                        Ext.Msg.alert(dSS._("Error"), param.failureMsg + ': ' +
                                      jsonData.message);
                    } else {
                        Ext.Msg.alert(dSS._("Error"), jsonData.message);
                    }
                } else {
                    ok = true;
                }
            } catch (error) {
                Ext.Msg.alert(dSS._("Error"), error);
            }

            if (param.maskEl !== undefined) {
                param.maskEl.unmask();
            }

            if (callback !== undefined) {
                callback(ok);
            }

            if (param.evt !== undefined) {
                this.fireEvent(param.evt, true); // scope set by ajax request
            }
        },
        failure: function(result, request) {
            var msg;
            try {
                var jsonData = Ext.util.JSON.decode(result.responseText);
                if (param.failureMsg !== undefined) {
                    msg = param.failureMsg + ': ' + jsonData.message;
                } else {
                    msg = jsonData.message;
                }
            } catch (err) {
                if (param.failureMsg !== undefined) {
                    msg = param.failureMsg + ': ' + err;
                } else {
                    msg = err;
                }
            }

            Ext.Msg.alert(dSS._("Error"), msg);
            if (param.maskEl !== undefined) {
                param.maskEl.unmask();
            }

            if (callback !== undefined) {
                callback(false);
            }

            if (param.evt !== undefined) {
                this.fireEvent(param.evt, false);
            }
        }
    });
};

dSS.util.getJokerGroup = function(groups) {
    var i;
    var group = null;
    var joker = false;

    for (i = 0; i < groups.length; i++) {
        group = parseInt(groups[i], 10);
        if (group !== dSS.config.groups.JOKER) {
            return group;
        } else {
            joker = true;
        }
    }

    if (joker === true) {
        return dSS.config.groups.JOKER;
    }

    return null;
};

dSS.util.updateJokerGroup = function(groups, oldGroup, newGroup) {
    var i;
    for (i = 0; i < groups.length; i++) {
        if (String(oldGroup) === groups[i]) {
            groups[i] = String(newGroup);
        }
    }
    return groups;
};

// device type, device class, return window icon (default false)
dSS.util.getDeviceIconCSS = function(type, cls, title, groups) {
    var css;
    var joker;

    if (groups === undefined) {
        groups = [];
    }

    if (cls === dSS.config.groups.JOKER) {
        var group = dSS.util.getJokerGroup(groups);
        if (group !== null) {
            joker = '_' + dSS.config.deviceColors[group];
        }
    }

    try {
        css = dSS.config.DeviceIconCSS[type][cls];
        if (joker !== undefined) {
            css = css + joker;
        }
    } catch (err) {
        if (cls !== undefined) {
            css = dSS.config.GenericDeviceIconCls[cls];
        }
    }

    if ((css !== undefined) && (title === true)) {
        css = css + dSS.config.TitleIconCSSExt;
    }

    return css;
};

dSS.util.UniqueArray = function(array, ints) {
  var i;
  var length = array.length;
  var helper = {};
  var result = [];

  for (i = 0; i < length; i++) {
      helper[array[i]] = 0;
  }

  for (i in helper) {
      if (ints === true) {
          result.push(parseInt(i, 10));
      } else {
          result.push(i);
      }
  }

  return result;
};

dSS.util.ResetDirtyField = function(field) {
    field.originalValue = field.getRawValue();
};

dSS.util.PushButtonToString = function(buttonID) {
    var dec = (buttonID & 0x0000000f);
    ret = "";

    switch (dec)
    {
        case dSS.config.deviceParams.PUSHBUTTON_DEVICE:
            ret = dSS._("Device");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP1:
            ret = dSS._("Area 1") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP5:
            ret = dSS._("Area 1") + ", " + dSS._("Scenes 12-14");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP2:
            ret = dSS._("Area 2") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP6:
            ret = dSS._("Area 2") + ", " + dSS._("Scenes 22-24");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP3:
            ret = dSS._("Area 3") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP7:
            ret = dSS._("Area 3") + ", " + dSS._("Scenes 32-34");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP4:
            ret = dSS._("Area 4") + ", " + dSS._("Scenes 2-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_GROUP8:
            ret = dSS._("Area 4") + ", " + dSS._("Scenes 42-44");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE0:
            ret = dSS._("Scenes 0-4");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE1:
            ret = dSS._("Scenes 10-14");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE2:
            ret = dSS._("Scenes 20-24");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE3:
            ret = dSS._("Scenes 30-34");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_ZONE4:
            ret = dSS._("Scenes 40-44");
            break;
        case dSS.config.deviceParams.PUSHBUTTON_HIGHLEVEL:
            ret = dSS._("server button");
            break;
        default:
            break;
    }

    return ret;
};

dSS.util.PushButtonModeToString = function(buttonMode, buttonIndex, buttonCount) {
    var dec = (buttonMode & 0x000000ff);
    ret = "";

    switch (dec)
    {
        default:
        case dSS.config.deviceParams.BUTTONINPUT_STANDARD:
        case dSS.config.deviceParams.BUTTONINPUT_TURBO:
            break;
            break;
        case dSS.config.deviceParams.BUTTONINPUT_SWITCH_STANDARD:
        case dSS.config.deviceParams.BUTTONINPUT_SWITCH_EDGE:
        case dSS.config.deviceParams.BUTTONINPUT_SWITCH_TIP:
            ret = dSS._("Switch");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT1:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT2:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT3:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_UP_WITH_INPUT4:
            ret = dSS._("2-way up");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT1:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT2:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT3:
        case dSS.config.deviceParams.BUTTONINPUT_2WAY_DW_WITH_INPUT4:
            ret = dSS._("2-way down");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_2WAY:
            ret = dSS._("2-way");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_1WAY:
            ret = dSS._("1-way");
            break;
        case dSS.config.deviceParams.BUTTONINPUT_SCENEDEVICE:
            ret = dSS._("Scene");
            break;
    }

    if(buttonCount > 1) {
        if(ret !==  "") {
            ret = dSS._("Line") + " " + (buttonIndex+1) + ", " + ret;
        } else {
            ret = dSS._("Line") + " " + (buttonIndex+1);
        }
    }

    return ret;
};

// "ok" paramter is needed in order to be able to chain the calls
dSS.util.setConfig = function(ok, dsid, cls, index, value, cb) {
    if (ok !== true) {
        if (cb !== undefined) {
            cb(ok);
        }
        return;
    }

    dSS.util.genericRequest({
            url: '/json/device/setConfig',
            params:
            {
                'class': cls,
                index: index,
                dsid: dsid,
                value: value
            },
            failureMsg: dSS._("Could not write device " +
                              "configuration"),
            scope: this
        },
        cb);
};

dSS.util.updateDSMLEDInfo = function(store, dsms) {
    Ext.each(dsms, function(dsm) {
        var index = store.find("dsid", new RegExp("^" + dsm.dsid +"$"));
        if (index > - 1) {
            var rec = store.getAt(index);
            rec.set('ledCode', dsm.ledcode);
            rec.set('ledStatus', dsm['status']);
        }
    });
    store.commitChanges();
};
